;---------------------------------------------------------------
;Theremin Paradox MX
;
;(c) Ilya Mamontov, 2019
;
;   (VPO 1.5 ),   --     (750 ),
;-   .
;-    24 .
;-  LCD (   P2[5]<->P2[6]   ) --   ,        P2[4]!!


; :
;
;   -   .     .     
;      "getkey" (  main_proc.asm). 
;        -           .
;    -  Counter8_1INT.asm  Timer16_1INT.asm
; Rx  Tx     -    COM    
;
;-----------------------------  -----------------------------
;!1 Bug in PSoC Designer 4.3: the directive "DWL label1" generates uncorrect code! Use the big endian format: "DW label1"
;!2       P2[4] - ..   LCD,   . 
;
;----------------------------- HISTORY ---------------------------
;01   Theremin Linearity Meter v.22:
;    100..10 (P0[7]).    24 (P1[4]).
;   LCD P2[0...6].                       P1[6]  P2[0...3]
;     ()  getkey,     .
;   Getkey    Timer16_1 ( 366,2 ,     SysClk).
;   vc3   UART (baud rate = 9600).     9 - .
;           .
;   getkey   10h,    UART  20h,    UART
;        FLASH   IMO ( 1  OSC_CR2).   "     
;         (    )       . " -  .    .
;      FLASH 20.  
;02     (   P1[7])
;         
;   P2[5]  P2[6]   LCD   --       (   ).
;   P1[6]    Pull Down    0,      .   
;   !    P2[4],     !
;04      .   -  .
;07    -    n-FET
;     - P0[2] 25  ( . )
;08      PGA .    -  P0[4].   --  Timer16_1INT.asm
;09         TDA7056A.   - P0[2] 25  
;14   ->  
;      itoa.asm -    ( 255   127)
;15      128.     .
;      .,   ,      .
;       24/16 = 1,5
;16   (  LCD) -    .  .       .
;17 .  -    -.
;18  .   .
;     (   ,     .)
;19   ,    .
;20  TimerCnt  TimerCntMax   1  (..     0,5 )
;        (  [noFreq],  )
;        (  PGA1  AGND)
;    .   - .
;21   ().    [an+0]...[an+10]     .
;22       1 (,  .  )!
;      Mute.  .   .
;23  +  -.
;24 AGND    P2[4].
;25      (  3   [modindex]).
;       .
;          ( ).
;26   (v1.26).   .
;---------------------------------------------------------------
include "m8c.inc"       ; part specific constants and macros
include "PSoCAPI.inc"   ; PSoC API definitions for all User Modules
include "flashblock.inc"
include "to_lcd_itoa.inc"


export _main
area bss(RAM)
T1:  BLK	1	;temp variables
T2:  BLK	2
T3:  BLK	1 
T4:  BLK	1
T5:  BLK	1
T6:  BLK	2	;

;------------------------------------------
area text(ROM,REL)
_main:

;---------------------- initialization of variables (by 0) ---------------------    
	mov [T1],vstart			;load start address in RAM
	mov A,0					;0
in1:mvi [T1],A				;0 -> RAM, [T1]++ 
	cmp [T1],vend			;end?
	jnz in1					;repeat
;----------------------- initialization of variables, misc. --------------------    
    mov [showmode],0x00		;  
    mov [ai],an				;load start address for dinamic indication
	mov [am],0b00100000		;load start scan mask
;	mov [scancode],0x01		;load start keyskan code 

;   mov [an+0],0b00000001	;LED 1		
;   mov [an+1],0b00100001	

    mov [an+2],0b00000001	;-
    mov [an+3],0b00000001	

;	mov [an+4],0b00000100	;LED  - "0"		
;   mov [an+5],0b01000100	

;   mov [an+6],0b10000000	;LED  - "V1"	
;   mov [an+7],0b10000010	

    mov [an+8],0b10000000	;LED Mute on
    mov [an+9],0b10000001	

;   mov [an+10],0b00000010	;LED 2		
;   mov [an+11],0b00100010	


;-------------------- initialization of variables, from FLASH ------------------    
;    !
	mov [T1],st				;load start address in RAM
	mov X,(<block)			;load start address of block
	mov [T2],(>block)		;
in2:mov A,[T2]				;load most significant addr byte
	romx					;to A
	mvi [T1],A				;to RAM, [bitCount]++ 
	inc X					;increment address 
	adc [T2],0				;
	cmp [T1],blockend		;end?
	jnz in2					;repeat
;--------------------------- end of initialization -----------------------  

;--------------------------- hardware configuration --------------------------  
    mov reg[PRT1DR],0b00000000 ; Port1: -  0  P1[6]       .  (   Pull Down!)
   												  ;-  0  P1[2]         (   "Mute")

    Counter8_1_EnableInt_M
    Counter8_1_Start_M				;start Counter8_1
    Counter8_2_Start_M				;start Counter8_2
    Counter8_3_Start_M				;start Counter8_3
    Counter8_4_Start_M				;start Counter8_4
    Timer16_1_Start_M				;start Timer16_1
 	Timer16_1_EnableInt_M
	
	mov A, 3
	lcall PGA_1_SetPower			;start PGA_1
	;mov A, 3
	;lcall PGA_2_SetPower			;start PGA_2
	mov A, 3
	lcall PGA_3_SetPower			;start PGA_3
	mov A, 3
	lcall CMPPRG_1_SetPower			;start CMPPRG_1
	mov A, 3
	lcall BPF_1_Start				;start BPF_1
	mov A, 3
	lcall LPF_1_Start				;start LPF_1
	mov A, 3
	lcall LPF_2_Start				;start LPF_2


	M8C_Stall 						;  stalling  (or reg[ASY_CR],1)

    M8C_SetBank1
    mov  A, reg[CLK_CR0]			;
    mov  reg[CLK_CR0], A			;Column Clock Re-Synchronization
	M8C_SetBank0

	
;	lcall  LCD_Start				;start LCD
;	toLCD_String 0,0,"Paradox MX      "
;	toLCD_String 1,0,"     v0.04      "


;	mov reg[TX8_1_CONTROL_REG], 1	;start TX8_1 with no parity
;	mov reg[RX8_1_CONTROL_REG], 1	;start RX8_1 with no parity

    or   reg[INT_MSK0], 20h
    mov  reg[INT_VC], 0 			; clear pending interrupts
    M8C_EnableGInt
	
;----   0.1 
	mov [TimerCntMax],37	;"37"-0.1s, "73"-0.2s, "183"-0.5s

;---    " "
	mov [showmode],3	;  
	mov [showcount],12	; 

;---    
	mov [pitchRange],0		;  0-bass, 1-, 2-, 3-
	mov [pitchVoice],0		;  0...11
	lcall setConfiguration	;  "pitchRange"  "pitchVoice"

;========================= main menu ==================================
loop:
	lcall getkey	;wait for key code
	tst [key],0x01
	jnz q1			;"S"
	tst [key],0x02
	jnz q2			;"-"
	tst [key],0x04
	jnz q3			;"+"
	tst [key],0x08
	jnz q4			;"Mute"
	jmp loop

q1:	;----------------------------- "S" key -----------------------------
	;   ( )
	mov [showmode],2	;  
	mov [showcount],15	; 
	jmp loop			;to main loop

q2:	;----------------------------- "-" key -----------------------------
	cmp [showmode],2	;    "S"
	jz .keyS

	;---    
	cmp [pitchVoice],0		;  
	jz .minimum1
		dec [pitchVoice]	; 
	.minimum1:
	jmp q31					;  

	;---     "S"
	.keyS:
	cmp [pitchRange],0		;  
	jz .minimum2
		dec [pitchRange]	; 
	.minimum2:
	jmp q32					;  

q3:	;----------------------------- "+"  key ----------------------------
	cmp [showmode],2		;    "S"
	jz keyS

	;---    
	cmp [pitchVoice],11		;  
	jnc .maximum1
		inc [pitchVoice]	; 
	.maximum1:
	
	q31:
	;  
	mov [showmode],1		;  
	mov [showcount],15		; 
	;   -       "pitchRange"  "pitchVoice"
	lcall setConfiguration
	jmp loop				;to main loop

	;---     "S"
	keyS:
	cmp [pitchRange],3		;  
	jnc .maximum2
		inc [pitchRange]	; 
	.maximum2:

	q32:
	;  
	mov [showmode],2		;   ()
	mov [showcount],15		;  ()
	;   -       "pitchRange"  "pitchVoice"
	lcall setConfiguration
	jmp loop				;to main loop

q4:	;-------------------------- "Mute" key -----------------------------
	xor [an+8],0b10000000	;  "Mute"
	mov A,0					; A -    ( 0  P1.2)   0  P1.6, ..         
	tst [an+8],0b10000000	;  "Mute"
	jnz .mute				;  "Mute"
    	or A,0b00000100 	; "Mute" -    ( 1  P1.2)
	.mute:
   	mov reg[PRT1DR],A 		;  Port1 (   )
	jmp loop				;to main loop
;----------------------------------------------------------------------



